@@ -44,6 +44,17 @@
   "Speechd-el user client customization."
   :group 'speechd-el)
 
+(defcustom speechd-speak-echo 'character
+  "Symbol determining how to read typed characters.
+It can have one of the following values:
+`character' -- read characters when they are typed
+`word' -- read only whole words when they are written
+`nil' -- don't echo anything on typing"
+  :type '(choice (const :tag "Characters" character)
+                 (const :tag "Words" word)
+                 (const :tag "Nothing" nil))
+  :group 'speechd-speak)
+
 (defcustom speechd-speak-deleted-char t
   "If non-nil, speak the deleted char, otherwise speak the adjacent char."
   :type 'boolean
@@ -576,7 +587,8 @@ to the end of the line."
   (interactive "P")
   (speechd-speak--interactive
    (let* ((inhibit-field-text-motion t)
-          (beg (if rest-only (point) (line-beginning-position)))
+          (in-isearch-p (and isearch-mode (not (called-interactively-p))))
+          (beg (if (and rest-only (not in-isearch-p)) (point) (line-beginning-position)))
           (end (line-end-position)))
      (when speechd-speak-separator-regexp
        (save-match-data
@@ -831,7 +843,18 @@ FUNCTION is invoked interactively."
 
 (defun* speechd-speak--previous-property-change (&optional (point (point))
                                                            (limit (point-min)))
-  (previous-char-property-change point limit))
+  ;; Let's be careful about isearch overlays not to cut texts in isearch
+  (let* ((i-overlays (and isearch-mode
+                          (intersection (overlays-at (- point 2))
+                                        (append (list isearch-overlay)
+                                                isearch-opened-overlays
+                                                isearch-lazy-highlight-overlays))))
+         (i-overlays-start (1- (apply #'min (1+ point) (mapcar #'overlay-start i-overlays)))))
+    (when (>= i-overlays-start limit)
+      ;; This may omit other property borders as well.  But in isearch we
+      ;; should retain enough context anyway.
+      (setq point i-overlays-start))
+    (previous-char-property-change point limit)))
 
 (defmacro speechd-speak--with-updated-text (&rest body)
   `(speechd-out-with-updated-text (speechd-speak--updated-text)
@@ -1310,7 +1333,9 @@ Only single characters are allowed in the keymap.")
 (defun speechd-speak--pre-command-hook ()
   (condition-case err
       (progn
-        (unless (memq this-command speechd-speak--dont-cancel-on-commands)
+        (unless (or (memq this-command speechd-speak--dont-cancel-on-commands)
+                    (and (eq this-command 'self-insert-command)
+                         (eq speechd-speak-echo 'word)))
           (speechd-out-cancel))
         (speechd-speak--set-command-start-info)
         (setq speechd-speak--last-report "")
@@ -1468,10 +1493,18 @@ Only single characters are allowed in the keymap.")
                                    (string-match "^.*$" text)
                                    (match-string 0 text))))))))
     (when (and self-insert
+               speechd-speak-echo
                (not (memq 'self-insert-command
                           speechd-speak-ignore-command-keys)))
-      (speechd-speak--with-updated-text
-        (speechd-speak--command-keys 'notification)))))
+      (case speechd-speak-echo
+        (word
+         (let ((point (point)))
+           (when (and (> point 1)
+                      (not (save-match-data (string-match "\\w" (buffer-substring (1- point) point)))))
+             (speechd-speak--text (buffer-substring (save-excursion (forward-word -1) (point)) point)))))
+        (character
+         (speechd-speak--with-updated-text
+          (speechd-speak--command-keys 'notification)))))))
 
 (speechd-speak--post-defun completions t t
   ;; *Completions* buffer
@@ -1677,10 +1710,6 @@ Only single characters are allowed in the keymap.")
 ;;; Other functions and packages
 
 
-(speechd-speak--command-feedback (isearch-search isearch-delete-char) after
-  (speechd-speak--text isearch-string)
-  (speechd-speak-read-line))
-
 (speechd-speak--command-feedback (occur-prev occur-next
 				  occur-mode-goto-occurence)
 				 after
