@@ -1,6 +1,6 @@
 ;;; speechd.el --- Library for accessing Speech Dispatcher
 
-;; Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008 Brailcom, o.p.s.
+;; Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2010 Brailcom, o.p.s.
 
 ;; Author: Milan Zamazal <pdm@brailcom.org>
 
@@ -50,8 +50,18 @@
   "SSIP interface."
   :group 'speechd-el)
 
+(defcustom speechd-connection-method 'unix-socket
+  "Connection method to Speech Dispatcher.
+Possible values are `unix-socket' for Unix domain sockets and
+`inet-socket' for Internet sockets on a given host and port (see
+`speechd-host' and `speechd-port' variables)."
+  :type 'symbol
+  :group 'speechd)
+
 (defcustom speechd-host (or (getenv "SPEECHD_HOST") "localhost")
-  "Name of the default host running speechd to connect to."
+  "Name of the default host running speechd to connect to.
+Value of this variable matters only when Internet sockets are
+used for communication with Speech Dispatcher."
   :type 'string
   :group 'speechd)
 
@@ -60,7 +70,9 @@
           (car (read-from-string (getenv "SPEECHD_PORT")))
         (error))
       6560)
-  "Default port of speechd."
+  "Default port of speechd.
+Value of this variable matters only when Internet sockets are
+used for communication with Speech Dispatcher."
   :type 'integer
   :group 'speechd)
 
@@ -72,6 +84,14 @@ variable is nil, Emacs talks to an SSIP server (Speech Dispatcher) directly."
                  (string :tag "spdsend binary"))
   :group 'speechd)
 
+(defcustom speechd-autospawn t
+  "If non-nil, Emacs will attempt to automatically start Speech Dispatcher.
+This means that if speechd-el gets a speech request and the
+Speech Dispatcher server is not running already, speechd-el will
+launch it."
+  :type 'string
+  :group 'speechd)
+
 (defcustom speechd-timeout 3
   "Number of seconds to wait for speechd response."
   :type 'integer
@@ -456,25 +476,36 @@ current voice."
 (defun speechd--use-spdsend ()
   (and (not speechd--advanced-apo) speechd-spdsend))
 
-(defun speechd--open-connection (host port)
+(defun speechd--open-connection (method host port socket-name)
   (if (speechd--use-spdsend)
       (let* ((answer (speechd--call-spdsend
                       (list "--open" host (format "%d" port))))
              (alen (and answer (length answer))))
-        (when (and alen (> alen 1))
-          (setq answer (substring answer 0 (1- alen))))
-        answer)
-    (let ((process (open-network-stream "speechd" nil host port)))
+	(when (and alen (> alen 1))
+	  (setq answer (substring answer 0 (1- alen))))
+	answer)
+    (let ((process
+           (cond
+            ((eq method 'unix-socket)
+             (make-network-process
+              :name "speechd" :family "local"
+              :remote (or socket-name
+                          (or (getenv "SPEECHD_SOCK")
+                              (expand-file-name
+                               "~/.speech-dispatcher/speechd.sock")))))
+            ((eq method 'inet-socket)
+             (open-network-stream "speechd" nil host port))
+            (t (error "Invalid communication method: `%s'" method)))))
       (when process
-        (set-process-coding-system process
-                                   speechd--coding-system
-                                   (if speechd--advanced-apo
-                                       speechd--coding-system
-                                     'raw-text))
-        (if (fboundp 'set-process-query-on-exit-flag)
-            (set-process-query-on-exit-flag process nil)
-          (process-kill-without-query process))
-        (set-process-filter process #'speechd--process-filter))
+	(set-process-coding-system process
+				   speechd--coding-system
+				   (if speechd--advanced-apo
+				       speechd--coding-system
+				     'raw-text))
+	(if (fboundp 'set-process-query-on-exit-flag)
+	    (set-process-query-on-exit-flag process nil)
+	  (process-kill-without-query process))
+	(set-process-filter process #'speechd--process-filter))
       process)))
 
 (defun speechd--close-connection (connection)
@@ -517,15 +548,23 @@ current voice."
      "Error on opening Speech Dispatcher connection")
 
 ;;;###autoload
-(defun* speechd-open (&optional host port &key quiet force-reopen)
-  "Open connection to Speech Dispatcher running on the given host and port.
+(defun* speechd-open (&optional method &key host port socket-name quiet force-reopen)
+  "Open connection to Speech Dispatcher using the given method.
 If the connection corresponding to the current `speechd-client-name' value
 already exists, close it and reopen again, with the same connection parameters.
 
-The optional arguments HOST and PORT identify the speechd server location.
-They can override default values stored in the variables `speechd-host' and
+Available methods are `unix-socket' and `inet-socket' for communication
+over Unix sockets and TCP sockets respectively.  Default is 'unix-socket'.
+
+The key arguments HOST and PORT are only relevant to the `inet-socket'
+communication method and identify the speechd server location.  They can
+override default values stored in the variables `speechd-host' and
 `speechd-port'.
 
+The SOCKET-NAME argument is only relevant to the `unix-socket' communication
+method and can override the default path to the Dispatcher's Unix socket for
+the given user.
+
 If the key argument QUIET is non-nil, don't report failures and quit silently.
 If the key argument FORCE-REOPEN is non-nil, try to reopen an existent
 connection even if it previously failed.
@@ -533,12 +572,16 @@ connection even if it previously failed.
 Return the opened connection on success, nil otherwise."
   (interactive)
   (let ((connection (gethash speechd-client-name speechd--connections)))
-    (let ((host (or host speechd-host))
+    (let ((method (or method speechd-connection-method))
+	  (host (or host speechd-host))
 	  (port (or port speechd-port)))
       (when connection
 	(speechd--close-connection connection)
 	(setq host (speechd--connection-host connection)
 	      port (speechd--connection-port connection)))
+      (when speechd-autospawn
+        (when (= (call-process "speech-dispatcher" nil nil nil "--spawn") 0)
+          (sleep-for 1)))
       (let* ((name speechd-client-name)
              (voice (cdr (assoc name speechd-connection-voices)))
              (default-parameters (append
@@ -560,7 +603,9 @@ Return the opened connection on success, nil otherwise."
 		  ((or (not connection)
 		       (not (speechd--connection-failure-p connection))
 		       force-reopen)
-                   (condition-case err (speechd--open-connection host port)
+                   (condition-case err (speechd--open-connection
+                                        (or method 'unix-socket)
+                                        host port socket-name)
                      (file-error
                       (setq connection-error err)
                       nil)))
